package info.aspirecn.cloud.yysj.controller;

import info.aspirecn.cloud.yysj.model.ResponseInfo;
import info.aspirecn.cloud.yysj.model.response.AccountInfo;
import info.aspirecn.cloud.yysj.model.response.ConsumptionInfo;
import info.aspirecn.cloud.yysj.model.response.OrderStatistics;
import info.aspirecn.cloud.yysj.service.AnalysisService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;
import java.util.List;

/**
 * 用户订单统计、消费信息、余额、产品消费比例等信息查询，适用于首页
 *
 * @author dingliqiang
 */
@Api(tags = "账户产品数据分析服务接口（适用于首页）")
@ApiResponses({
        @ApiResponse(code = 200, message = "成功处理请求"),
        @ApiResponse(code = 400, message = "没有找到该用户信息"),
        @ApiResponse(code = 401, message = "没有权限访问该服务"),
        @ApiResponse(code = 403, message = "权限不足无法访问该服务"),
        @ApiResponse(code = 404, message = "未发现该微服务"),
        @ApiResponse(code = 500, message = "服务器内部错误")
})
@Slf4j
@RestController
public class AnalysisController {

    @Autowired
    private AnalysisService analysisService;

    /**
     * 主订单统计信息接口
     * @param userId 用户ID
     * @return 主订单统计信息
     */
    @ApiOperation(value = "主订单统计信息", notes = "获取账户最新 15 天订单统计信息。")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户ID", dataType = "string", required = false, paramType = "header")
    })
    @GetMapping("/statisticsInfo")
    public ResponseInfo getOrderStatistics(@RequestHeader(required = true) String userId) {
        List<OrderStatistics> statisticsList = analysisService.getOrderStatistics(userId);
        return new ResponseInfo().setData(statisticsList).setMessage("请求成功");
    }

    /**
     * 产品消费信息接口
     * @param userId 用户ID
     * @return 产品消费信息
     */
    @ApiOperation(value = "产品消费信息", notes = "获取账户当月订购产品的消费及占比信息。")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户ID", dataType = "string", required = true, paramType = "header")
    })
    @GetMapping("/consumptionInfo")
    public ResponseInfo getConsumptionInfo(@RequestHeader(required = true) String userId) {
        List<ConsumptionInfo> consumptionList = analysisService.getConsumptionInfo(userId);
        return new ResponseInfo().setData(consumptionList).setMessage("请求成功");
    }

    /**
     * 账户信息接口
     * @param userId 用户ID
     * @return 账户信息
     */
    @ApiOperation(value = "账户信息", notes = "获取用户当日消费金额和账户余额。")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户ID", dataType = "string", required = true, paramType = "header")
    })
    @GetMapping("/accountInfo")
    public ResponseInfo getAccountInfo(@RequestHeader(required = true) String userId) {
        AccountInfo accountInfo = analysisService.getAccountInfo(userId);
        return new ResponseInfo().setData(accountInfo).setMessage("请求成功");
    }

}
