package info.aspirecn.cloud.yysj.service.impl;

import info.aspirecn.cloud.yysj.dao.es.YysjGatewayFirstIndex;
import info.aspirecn.cloud.yysj.dao.es.YysjGatewaySecondIndex;
import info.aspirecn.cloud.yysj.exception.UserNotFountException;
import info.aspirecn.cloud.yysj.model.entity.User;
import info.aspirecn.cloud.yysj.model.entity.UserProduce;
import info.aspirecn.cloud.yysj.model.remote.SjjhAccountInfo;
import info.aspirecn.cloud.yysj.model.response.AccountInfo;
import info.aspirecn.cloud.yysj.model.response.ConsumptionInfo;
import info.aspirecn.cloud.yysj.model.response.OrderStatistics;
import info.aspirecn.cloud.yysj.remote.FindUserInfoFeign;
import info.aspirecn.cloud.yysj.service.AnalysisService;
import info.aspirecn.cloud.yysj.service.DbService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.*;

/**
 * 数据分析,用于首页
 *
 * @author dingliqiang
 */
@Slf4j
@Service
public class AnalysisServiceImpl implements AnalysisService {

    @Autowired
    private FindUserInfoFeign findUserInfoFeign;

    @Autowired
    private DbService dbService;

    @Autowired
    private YysjGatewayFirstIndex yysjGatewayFirstIndex;

    @Autowired
    private YysjGatewaySecondIndex yysjGatewaySecondIndex;

    /**
     * 统计云验平台用户调用总量、车辆、运单、司机信息
     *
     * @param userId 云验数据平台用户ID
     * @return 总量、车辆、运单、司机信息
     */
    @Override
    public List<OrderStatistics> getOrderStatistics(String userId) {
        return yysjGatewayFirstIndex.getTotalConsumptionAmount(userId);
    }

    /**
     * @param userId 云验数据平台用户ID
     * @return 账户当月订购产品的消费及占比信息
     */
    @Override
    public List<ConsumptionInfo> getConsumptionInfo(String userId) {
        List<ConsumptionInfo> consumptionList = new ArrayList<>();
        // 获取用户信息
        User user = dbService.getUserInfo(userId);
        if (user == null) {
            throw new UserNotFountException("无法查询到用户 " + userId + " 信息");
        }
        // 设置云验数据平台产品ID列表
        List<Integer> yysjProduceIds = new ArrayList<>();
        // 获取用户关联的产品信息
        List<UserProduce> userProduceList = dbService.getUserProduceList(userId);
        // 获取数据交互中的全部产品ID
        for (UserProduce userProduce : userProduceList) {
            yysjProduceIds.add(userProduce.getProduceId());
        }
        // 获取云验平台用户当月订购产品的消费及占比信息
        Map<Integer, ConsumptionInfo> consumptionInfoMap = yysjGatewaySecondIndex.getConsumptionInfo(userId,
                userProduceList, yysjProduceIds.toArray(new Integer[yysjProduceIds.size()]));
        // 根据产品ID到产品表中查询对应的产品名称
        for (Map.Entry<Integer, ConsumptionInfo> entry : consumptionInfoMap.entrySet()) {
            // 从数据库中查询产品名称
            String produceName = dbService.getProduceInfo(entry.getValue().getProductId()).getProduceName();
            // 设置产品名称
            entry.getValue().setProductName(produceName);
            // 处理价格单位（从厘转换为元）
            String price = priceProcess(entry.getValue().getConsumptionAmount());
            entry.getValue().setConsumptionAmount(price);
            // 添加到列表
            consumptionList.add(entry.getValue());
        }
        return consumptionList;
    }

    /**
     * 获取用户当日消费金额和账户余额
     *
     * @param userId 云验数据平台用户ID
     * @return 当日消费金额、用户余额
     */
    @Override
    public AccountInfo getAccountInfo(String userId) {
        // 设置返回对象
        AccountInfo accountInfo = new AccountInfo();
        // 获取用户信息
        User user = dbService.getUserInfo(userId);
        if (user == null) {
            throw new UserNotFountException("无法查询到用户 " + userId + " 信息");
        }
        // 从 ES 查询账户当日消费数据
        Long dailyConsumption = yysjGatewaySecondIndex.getAccountConsumptionAmount(userId);
        accountInfo.setDailyConsumption(priceProcess(String.valueOf(dailyConsumption)));
        // 从数据库查询账户余额
        SjjhAccountInfo sjjhAccountInfo = findUserInfoFeign.getUserAccount(user.getSjjhUserId());
        accountInfo.setAccountBalance(priceProcess(sjjhAccountInfo.getBalance()));
        return accountInfo;
    }

    /**
     * 处理价格单位
     */
    private String priceProcess(String price) {
        // 获取签名部分
        if (price.length() == 1) {
            return "0.00" + price;
        } else if (price.length() == 2) {
            return "0.0" + price;
        } else if (price.length() == 3) {
            return "0." + price;
        }
        // 获取签名部分
        String pricePre = StringUtils.substring(price, 0, price.length() - 3);
        // 获取后面部分
        String pricePost = StringUtils.substring(price, price.length() - 3, price.length());
        // 拼接
        return pricePre + "." + pricePost;
    }

}
